unit hiTranslator;

interface

uses
  Windows, KOL, Share, CodePages;

type
  THITranslator = object
    private
      function ReadState: string;
      procedure LoadStrings(Lst: PKOLStrList);
      procedure LoadListFromFile;
    public
      _prop_Lang: string;
      _prop_LangWords: PKOLStrListEx;
      _prop_Place: Byte;
      _prop_Key: string;
      _prop_Section: string;
      _prop_Value: string;
      _prop_LangsDir: string;
      
      procedure Init;
  end;


implementation


const
  RootRegKeys: array [0..6{13}] of
    record
      KeyName: string;
      KeyH: HKEY;
    end
   =
   (
     {(KeyName: 'HKCR'; KeyH: HKEY_CLASSES_ROOT),
     (KeyName: 'HKCU'; KeyH: HKEY_CURRENT_USER),
     (KeyName: 'HKLM'; KeyH: HKEY_LOCAL_MACHINE),
     (KeyName: 'HKU'; KeyH: HKEY_USERS),
     (KeyName: 'HKPD'; KeyH: HKEY_PERFORMANCE_DATA),
     (KeyName: 'HKCC'; KeyH: HKEY_CURRENT_CONFIG),
     (KeyName: 'HKDD'; KeyH: HKEY_DYN_DATA),}

     (KeyName: 'HKEY_CLASSES_ROOT'; KeyH: HKEY_CLASSES_ROOT),
     (KeyName: 'HKEY_CURRENT_USER'; KeyH: HKEY_CURRENT_USER),
     (KeyName: 'HKEY_LOCAL_MACHINE'; KeyH: HKEY_LOCAL_MACHINE),
     (KeyName: 'HKEY_USERS'; KeyH: HKEY_USERS),
     (KeyName: 'HKEY_PERFORMANCE_DATA'; KeyH: HKEY_PERFORMANCE_DATA),
     (KeyName: 'HKEY_CURRENT_CONFIG'; KeyH: HKEY_CURRENT_CONFIG),
     (KeyName: 'HKEY_DYN_DATA'; KeyH: HKEY_DYN_DATA)
   );

function RegRootKeyByName(RootKeyName: string): HKey;
var
  I: Byte;
begin
  Result := 0;
  RootKeyName := UpperCase(RootKeyName);
  for I := 0 to High(RootRegKeys) do
  begin
    with RootRegKeys[I] do
    begin
      if KeyName = RootKeyName then
      begin
        Result := KeyH;
        Exit;
      end;
    end;
  end;
end;





function THITranslator.ReadState: string;
var
  Key: HKey;
  Ini: PIniFile;
begin
  if _prop_Place = 0 then
  begin
    Key := RegRootKeyByName(_prop_Key);
    if Key <> 0 then
    begin
      Key := RegKeyOpenRead(Key, _prop_Section);
      Result := RegKeyGetStr(Key, _prop_Value);
      RegKeyClose(Key);
    end;
  end
  else
  begin
    Ini := OpenIniFile(GetStartDir + _prop_Key);
    Ini.section := _prop_Section;
    Result := Ini.ValueString(_prop_Value, '');
    Ini.Free;
  end;
end;

procedure THITranslator.LoadStrings(Lst: PKOLStrList);
var
  I: Integer;
  S: string;
  Sources: PKOLStrList;
  Dests: PKOLStrList;
begin
  Sources := NewKOLStrList;
  Dests := NewKOLStrList;
  for I := 0 to Lst.Count - 1 do
  begin
    S := Lst.Items[I];
    Sources.Add(GetTok(S, '|')); 
    Dests.Add(S);
  end;
  Translator.SetTranslate(Sources, Dests);
end;

procedure THITranslator.LoadListFromFile;
var
  Files: PDirList;
  TransL: PKOLStrList;
  I: Integer;
  LCode: string;
begin
  LCode := ReadState;
  Files := NewDirList(GetStartDir + _prop_LangsDir, '*.lng', FILE_ATTRIBUTE_NORMAL);
  TransL := NewKOLStrList;
  for I := 0 to Files.Count - 1 do
    if ExtractFileNameWOExt(Files.Items[I].cFileName) = LCode then
    begin
      //TransL.LoadFromFile(GetStartDir + _prop_LangsDir + '\' + Files.Items[I].cFileName);
      StrListLoadFromFile(TransL, GetStartDir + _prop_LangsDir + '\' + Files.Items[I].cFileName, CP_UTF8);
      LoadStrings(TransL);
      Break; 
    end; 
  Files.free;
  TransL.Free;
end;

procedure THITranslator.Init;
var
  Sources: PKOLStrList;
  Dests: PKOLStrList;
  I: Integer;
begin
  if _prop_Lang = '' then
    LoadListFromFile
  else
    if ReadState = _prop_Lang then
    begin
      Sources := NewKOLStrList;
      Dests := NewKOLStrList;
      for I := 0 to _prop_LangWords.Count - 1 do
      begin
        Sources.Add(_prop_LangWords.Items[I]);
        Dests.Add(PChar(Pointer(_prop_LangWords.Objects[I])));
      end;
      Translator.SetTranslate(Sources, Dests);
    end;
end;

end.
